@@ -20,6 +20,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <assert.h>
 #include <sys/resource.h>
 
 #include "byteorder.h"
@@ -46,33 +47,39 @@ long int myatoi( const char *str )
   return ret;
 }
 
-static void * aligned_alloc( int len )
+AlignedBuffer::AlignedBuffer( size_t len, const char *data )
+  : m_len( len ), m_allocated( NULL ), m_data( NULL )
 {
-  void *ptr = NULL;
-
 #if defined(HAVE_POSIX_MEMALIGN)
-  if( (0 != posix_memalign( (void **)&ptr, 16, len )) || (ptr == NULL) ) {
+  if ( ( 0 != posix_memalign( &m_allocated, 16, len ) )
+      || ( m_allocated == NULL ) ) {
     throw std::bad_alloc();
   }
+  m_data = (char *) m_allocated;
+
 #else
-  // Some platforms will align malloc. Let's try that first.
-  if( ! (ptr = malloc(len)) ) {
+  /* malloc() a region 15 bytes larger than we need, and find
+     the aligned offset within. */
+  m_allocated = malloc( 15 + len );
+  if ( m_allocated == NULL ) {
     throw std::bad_alloc();
   }
-  if( (uintptr_t)ptr & 0xF ) {
-    // The pointer wasn't 16-byte aligned, so try again with valloc
-    free(ptr);
-    if( ! (ptr = valloc(len)) ) {
-      throw std::bad_alloc();
-    }
-    if( (uintptr_t)ptr & 0xF ) {
-      free(ptr);
-      throw std::bad_alloc();
-    }
+
+  uintptr_t iptr = (uintptr_t) m_allocated;
+  if ( iptr & 0xF ) {
+    iptr += 16 - ( iptr & 0xF );
   }
+  assert( !( iptr & 0xF ) );
+  assert( iptr >= (uintptr_t) m_allocated );
+  assert( iptr <= ( 15 + (uintptr_t) m_allocated ) );
+
+  m_data = (char *) iptr;
+
 #endif /* !defined(HAVE_POSIX_MEMALIGN) */
 
-  return ptr;
+  if ( data ) {
+    memcpy( m_data, data, len );
+  }
 }
 
 Base64Key::Base64Key( string printable_key )
@@ -131,13 +138,12 @@ string Base64Key::printable_key( void ) const
 }
 
 Session::Session( Base64Key s_key )
-  : key( s_key ), ctx( NULL ), blocks_encrypted( 0 )
+  : key( s_key ), ctx_buf( ae_ctx_sizeof() ),
+    ctx( (ae_ctx *)ctx_buf.data() ), blocks_encrypted( 0 ),
+    plaintext_buffer( RECEIVE_MTU ),
+    ciphertext_buffer( RECEIVE_MTU ),
+    nonce_buffer( Nonce::NONCE_LEN )
 {
-  ctx = ae_allocate( NULL );
-  if ( ctx == NULL ) {
-    throw CryptoException( "Could not allocate AES-OCB context." );
-  }
-
   if ( AE_SUCCESS != ae_init( ctx, key.data(), 16, 12, 16 ) ) {
     throw CryptoException( "Could not initialize AES-OCB context." );
   }
@@ -148,8 +154,6 @@ Session::~Session()
   if ( ae_clear( ctx ) != AE_SUCCESS ) {
     throw CryptoException( "Could not clear AES-OCB context." );
   }
-
-  ae_free( ctx );
 }
 
 Nonce::Nonce( uint64_t val )
@@ -193,26 +197,21 @@ string Session::encrypt( Message plaintext )
   const size_t pt_len = plaintext.text.size();
   const int ciphertext_len = pt_len + 16;
 
-  char *ciphertext = (char *)aligned_alloc( ciphertext_len );
-  char *pt = (char *)aligned_alloc( pt_len );
+  assert( (size_t)ciphertext_len <= ciphertext_buffer.len() );
+  assert( pt_len <= plaintext_buffer.len() );
 
-  memcpy( pt, plaintext.text.data(), plaintext.text.size() );
-
-  if ( (uint64_t( plaintext.nonce.data() ) & 0xf) != 0 ) {
-    throw CryptoException( "Bad alignment." );
-  }
+  memcpy( plaintext_buffer.data(), plaintext.text.data(), pt_len );
+  memcpy( nonce_buffer.data(), plaintext.nonce.data(), Nonce::NONCE_LEN );
 
   if ( ciphertext_len != ae_encrypt( ctx,                                     /* ctx */
-				     plaintext.nonce.data(),                  /* nonce */
-				     pt,                                      /* pt */
+				     nonce_buffer.data(),                     /* nonce */
+				     plaintext_buffer.data(),                 /* pt */
 				     pt_len,                                  /* pt_len */
 				     NULL,                                    /* ad */
 				     0,                                       /* ad_len */
-				     ciphertext,                              /* ct */
+				     ciphertext_buffer.data(),                /* ct */
 				     NULL,                                    /* tag */
 				     AE_FINALIZE ) ) {                        /* final */
-    free( pt );
-    free( ciphertext );
     throw CryptoException( "ae_encrypt() returned error." );
   }
 
@@ -235,14 +234,10 @@ string Session::encrypt( Message plaintext )
      client use the same key, so we actually need to die after 2^47 blocks.
   */
   if ( blocks_encrypted >> 47 ) {
-    free( pt );
-    free( ciphertext );
     throw CryptoException( "Encrypted 2^47 blocks.", true );
   }
 
-  string text( (char *)ciphertext, ciphertext_len );
-  free( pt );
-  free( ciphertext );
+  string text( ciphertext_buffer.data(), ciphertext_len );
 
   return plaintext.nonce.cc_str() + text;
 }
@@ -263,43 +258,56 @@ Message Session::decrypt( string ciphertext )
     exit( 1 );
   }
 
-  Nonce __attribute__((__aligned__ (16))) nonce( str, 8 );
-  char *body = (char *)aligned_alloc( body_len );
-  memcpy( body, str + 8, body_len );
+  assert( (size_t)body_len <= ciphertext_buffer.len() );
+  assert( (size_t)pt_len <= plaintext_buffer.len() );
 
-  char *plaintext = (char *)aligned_alloc( pt_len );
+  Nonce nonce( str, 8 );
+  memcpy( ciphertext_buffer.data(), str + 8, body_len );
+  memcpy( nonce_buffer.data(), nonce.data(), Nonce::NONCE_LEN );
 
-  if ( pt_len != ae_decrypt( ctx,               /* ctx */
-			     nonce.data(),      /* nonce */
-			     body,              /* ct */
-			     body_len,          /* ct_len */
-			     NULL,              /* ad */
-			     0,                 /* ad_len */
-			     plaintext,         /* pt */
-			     NULL,              /* tag */
-			     AE_FINALIZE ) ) {  /* final */
-    free( plaintext );
-    free( body );
+  if ( pt_len != ae_decrypt( ctx,                      /* ctx */
+			     nonce_buffer.data(),      /* nonce */
+			     ciphertext_buffer.data(), /* ct */
+			     body_len,                 /* ct_len */
+			     NULL,                     /* ad */
+			     0,                        /* ad_len */
+			     plaintext_buffer.data(),  /* pt */
+			     NULL,                     /* tag */
+			     AE_FINALIZE ) ) {         /* final */
     throw CryptoException( "Packet failed integrity check." );
   }
 
-  Message ret( nonce, string( plaintext, pt_len ) );
-  free( plaintext );
-  free( body );
+  Message ret( nonce, string( plaintext_buffer.data(), pt_len ) );
 
   return ret;
 }
 
+static rlim_t saved_core_rlimit;
+
 /* Disable dumping core, as a precaution to avoid saving sensitive data
    to disk. */
 void Crypto::disable_dumping_core( void ) {
   struct rlimit limit;
-  limit.rlim_cur = 0;
-  limit.rlim_max = 0;
-  if ( 0 != setrlimit( RLIMIT_CORE, &limit ) ) {
+  if ( 0 != getrlimit( RLIMIT_CORE, &limit ) ) {
     /* We don't throw CryptoException because this is called very early
        in main(), outside of 'try'. */
+    perror( "getrlimit(RLIMIT_CORE)" );
+    exit( 1 );
+  }
+
+  saved_core_rlimit = limit.rlim_cur;
+  limit.rlim_cur = 0;
+  if ( 0 != setrlimit( RLIMIT_CORE, &limit ) ) {
     perror( "setrlimit(RLIMIT_CORE)" );
     exit( 1 );
   }
 }
+
+void Crypto::reenable_dumping_core( void ) {
+  /* Silent failure is safe. */
+  struct rlimit limit;
+  if ( 0 == getrlimit( RLIMIT_CORE, &limit ) ) {
+    limit.rlim_cur = saved_core_rlimit;
+    setrlimit( RLIMIT_CORE, &limit );
+  }
+}
