@@ -38,15 +38,13 @@
 #include <util.h>
 #endif
 
-extern "C" {
-#include "selfpipe.h"
-}
-
+#include "sigfd.h"
 #include "swrite.h"
 #include "completeterminal.h"
 #include "user.h"
 #include "terminaloverlay.h"
 #include "locale_utils.h"
+#include "fatal_assert.h"
 
 const int ITERATIONS = 100000;
 
@@ -64,7 +62,7 @@ int main( void )
 
   /* Adopt native locale */
   set_native_locale();
-  assert_utf8_locale();
+  fatal_assert( is_utf8_locale() );
 
   for ( int i = 0; i < ITERATIONS; i++ ) {
     /* type a character */
