@@ -42,6 +42,7 @@
 #include "parser.h"
 #include "swrite.h"
 #include "locale_utils.h"
+#include "fatal_assert.h"
 
 const size_t buf_size = 1024;
 
@@ -57,7 +58,7 @@ int main( int argc __attribute__((unused)),
   struct termios saved_termios, raw_termios, child_termios;
 
   set_native_locale();
-  assert_utf8_locale();
+  fatal_assert( is_utf8_locale() );
 
   if ( tcgetattr( STDIN_FILENO, &saved_termios ) < 0 ) {
     perror( "tcgetattr" );
