@@ -266,9 +266,8 @@ void Connection::send( string s )
   ssize_t bytes_sent = sendto( sock, p.data(), p.size(), 0,
 			       (sockaddr *)&remote_addr, sizeof( remote_addr ) );
 
-  if ( bytes_sent == static_cast<ssize_t>( p.size() ) ) {
-    return;
-  } else {
+  if ( (!server) /* Server treats all sendto()s as successful. */
+       && (bytes_sent != static_cast<ssize_t>( p.size() )) ) {
     throw NetworkException( "sendto", errno );
   }
 }
@@ -277,20 +276,20 @@ string Connection::recv( void )
 {
   struct sockaddr_in packet_remote_addr;
 
-  char buf[ RECEIVE_MTU ];
+  char buf[ Session::RECEIVE_MTU ];
 
   socklen_t addrlen = sizeof( packet_remote_addr );
 
-  ssize_t received_len = recvfrom( sock, buf, RECEIVE_MTU, 0, (sockaddr *)&packet_remote_addr, &addrlen );
+  ssize_t received_len = recvfrom( sock, buf, Session::RECEIVE_MTU, 0, (sockaddr *)&packet_remote_addr, &addrlen );
 
   if ( received_len < 0 ) {
     throw NetworkException( "recvfrom", errno );
   }
 
-  if ( received_len > RECEIVE_MTU ) {
+  if ( received_len > Session::RECEIVE_MTU ) {
     char buffer[ 2048 ];
     snprintf( buffer, 2048, "Received oversize datagram (size %d) and limit is %d\n",
-	      static_cast<int>( received_len ), RECEIVE_MTU );
+	      static_cast<int>( received_len ), Session::RECEIVE_MTU );
     throw NetworkException( buffer, errno );
   }
 
@@ -427,22 +426,9 @@ uint64_t Connection::timeout( void ) const
   return RTO;
 }
 
-class Socket {
-public:
-  int fd;
-
-  Socket( int domain, int type, int protocol )
-    : fd( socket( domain, type, protocol ) )
-  {
-    if ( fd < 0 ) {
-      throw NetworkException( "socket", errno );
-    }
+Connection::~Connection()
+{
+  if ( close( sock ) < 0 ) {
+    throw NetworkException( "close", errno );
   }
-
-  ~Socket()
-  {
-    if ( close( fd ) < 0 ) {
-      throw NetworkException( "close", errno );
-    }
-  }
-};
+}
