@@ -191,13 +191,20 @@ void Ctrl_HT( Framebuffer *fb, Dispatcher *dispatch __attribute((unused)) )
 {
   int col = fb->ds.get_next_tab();
   if ( col == -1 ) { /* no tabs, go to end of line */
-    fb->ds.move_col( fb->ds.get_width() - 1 );
+  /* A horizontal tab is the only operation that (1) can keep the wrap
+     flag but (2) starts a new grapheme. */
+    if ( fb->ds.get_cursor_col() == fb->ds.get_width() - 1 ) {
+      fb->ds.move_col( fb->ds.get_width() - 1, false );
+      fb->ds.move_col( 1, true, true );
+    } else {
+      fb->ds.move_col( fb->ds.get_width() - 1, false );
+    }
   } else {
-    fb->ds.move_col( col );
+    fb->ds.move_col( col, false );
   }
 }
 
-static Function func_Ctrl_HT( CONTROL, "\x09", Ctrl_HT );
+static Function func_Ctrl_HT( CONTROL, "\x09", Ctrl_HT, false );
 
 /* horizontal tab set */
 void Ctrl_HTS( Framebuffer *fb, Dispatcher *dispatch __attribute((unused)) )
@@ -216,6 +223,7 @@ void CSI_TBC( Framebuffer *fb, Dispatcher *dispatch )
     fb->ds.clear_tab( fb->ds.get_cursor_col() );    
     break;
   case 3: /* clear all tab stops */
+    fb->ds.clear_default_tabs();
     for ( int x = 0; x < fb->ds.get_width(); x++ ) {
       fb->ds.clear_tab( x );
     }
@@ -273,8 +281,9 @@ void CSI_DECRM( Framebuffer *fb, Dispatcher *dispatch )
   }
 }
 
-static Function func_CSI_DECSM( CSI, "?h", CSI_DECSM );
-static Function func_CSI_DECRM( CSI, "?l", CSI_DECRM );
+/* These functions don't clear wrap state. */
+static Function func_CSI_DECSM( CSI, "?h", CSI_DECSM, false );
+static Function func_CSI_DECRM( CSI, "?l", CSI_DECRM, false );
 
 static bool *get_ANSI_mode( int param, Framebuffer *fb ) {
   switch ( param ) {
@@ -315,6 +324,12 @@ void CSI_DECSTBM( Framebuffer *fb, Dispatcher *dispatch )
   int top = dispatch->getparam( 0, 1 );
   int bottom = dispatch->getparam( 1, fb->ds.get_height() );
 
+  if ( (bottom <= top)
+       || (top > fb->ds.get_height())
+       || (top == 0 && bottom == 1) ) {
+    return; /* invalid, xterm ignores */
+  }
+
   fb->ds.set_scrolling_region( top - 1, bottom - 1 );
   fb->ds.move_row( 0 );
   fb->ds.move_col( 0 );
@@ -499,12 +514,29 @@ static Function func_CSI_DECSTR( CSI, "!p", CSI_DECSTR );
 void Dispatcher::OSC_dispatch( const Parser::OSC_End *act, Framebuffer *fb )
 {
   if ( OSC_string.size() >= 2 ) {
-    if ( (OSC_string[ 0 ] == L'0')
+    if ( ( (OSC_string[ 0 ] == L'0')
+	   || (OSC_string[ 0 ] == L'1')
+	   || (OSC_string[ 0 ] == L'2') )
 	 && (OSC_string[ 1 ] == L';') ) {
       std::deque<wchar_t> newtitle( OSC_string.begin(), OSC_string.end() );
       newtitle.erase( newtitle.begin() );
       newtitle.erase( newtitle.begin() );
-      fb->set_window_title( newtitle );
+
+      switch ( OSC_string[ 0 ] ) {
+      case L'0':
+	fb->set_icon_name( newtitle );
+	fb->set_window_title( newtitle );
+	break;
+      case L'1':
+	fb->set_icon_name( newtitle );
+	break;
+      case L'2':
+	fb->set_window_title( newtitle );
+	break;
+      default:
+	break;
+      }
+
       act->handled = true;
     }
   }
